#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé Nov 2024
@ auteur: david ALBERTO (www.astrolabe-science.fr)
"""
import matplotlib.pyplot as plt
import pandas as pd
from mendeleev.fetch import fetch_table

#  ----paramètres à personnaliser ----------------------------

plt.rcParams["font.family"] = "Roboto Condensed"  # ou autre police installée

    # couleur des modes de désintégration :

carac_modes = {
    'label':    [ r'$\alpha$', r'$\beta^-$', r'$\beta^+$', 'capt. élect.', 'fission spont.',
                 'ém. neutron', 'ém. proton'],
    'couleur' : ['orange', 'beige', 'yellowgreen','teal', 'mediumslateblue',
                  'tomato', 'gold'],
    # 'couleur' : ['#a63d4c', '#DBD3D1', '#A1B4A5','#385752', 'steelblue',
    #              'tomato', '#6c9836'],
    'caractere' : ['A', 'B-', 'B+', 'EC', 'SF', 'n', 'p']
    }
couleur_stables = 'black'
# création d'un DataFrame avec les couleurs et autres infos sur les modes :
decaymodes_carac_modes = pd.DataFrame(carac_modes)

# propriétés communes pour le formatage des points :
prop_points = dict(
    s=7,  # taille des points, à ajuster à la dimension du graphique
    ec='k',  # couleur de la bordure
    lw=0.1,  # épaisseur de la bordure
    # marker='s'  # marqueur carré
    )

#  -----------------------------------------------------------
#  -------- import des données des éléments ------------------

decaymodes = fetch_table('isotopedecaymodes')  # modes de désintégration de nucléides
iso = fetch_table('isotopes')  # n° atomique et nb de masse des nucléides

# maintien des colonnes utiles :
iso = iso[['id', 'atomic_number', 'mass_number','is_radioactive', 'abundance']]
    # suppression des colonnes inutiles :
decaymodes = decaymodes.drop([
    'is_allowed_not_observed',
    'relation',
    'is_observed_intensity_unknown'
    ], axis=1)

    # suppression des nucléides dont le mode est possible mais inobservé :
# decaymodes = decaymodes[decaymodes.is_allowed_not_observed != True]
    # suppression nucléides avec intensité inconnue :
# decaymodes = decaymodes[decaymodes.is_observed_intensity_unknown != True]

# decaymodes = decaymodes.drop(['is_observed_intensity_unknown'], axis=1)

groupes_modes = decaymodes.groupby('mode').count().sort_values(by='id', ascending=False)

"""
Le tableau 'iso' contient les n° atomiques et nb de masse des isotopes, alors
que le tableau decaymodes contient leurs modes de désintégration.
La correspondance se fait grâce à une colonne 'id' et une colonne 'isotope_id'.
Ci-après, on crée une nouvelle colonne 'ident' dans chaque tableau, contenant les
infos communes pour identifier les nucléides. Ensuite on fusionne les tableaux
en faisant correspondre ces colonnes.
"""
    # ajout de 2 colonnes communes dans les tableaux, pour préparer la fusion :
decaymodes['ident'] = decaymodes.isotope_id
iso['ident'] = iso.id

    # fusion des deux tableaux, pour
merge = pd.merge(decaymodes, iso, on='ident')  # tableau fusionné
merge['N'] = merge.mass_number - merge.atomic_number  # ajout colonne nb neutrons

Zmax = iso.atomic_number.max()
# Nmax = iso.mass_number.max()
Nmax = 180
# ------------------------------------------------------------
#  ------------ définition des fonctions --------------------

def points(mode, axe):
    """
    mode : string : code du mode de désintégration.
    Trace le nuage de points des nucléides pour le mode indiqué, avec
    la légende.
    """
    selection = merge.loc[merge['mode'] == mode]
    x = selection.atomic_number
    y = selection.N
    couleur = decaymodes_carac_modes.loc[decaymodes_carac_modes.caractere == mode, 'couleur'].values[0]
    label = decaymodes_carac_modes.loc[decaymodes_carac_modes.caractere == mode, 'label'].values[0]
    axe.scatter(x, y, c=couleur, label=label, **prop_points)

def points_stables(axe):
    """
    Trace le nuage de points des isotopes stables.
    Deux critères à réunir pour repérer les stables :
        - non radioactifs
        - abondance minimale (valeur arbitraire). Sinon, on a des noyaux
        stables très loin de la courbe de stabilité centrale.
    """
    stables = iso[(iso.is_radioactive == False)
                  &
                  (iso.abundance > 1e-5)]
    x = stables.atomic_number
    y = stables.mass_number - stables.atomic_number
    axe.scatter(x, y, c=couleur_stables, **prop_points, label='stables')

#  ------------- paramètres du graphique --------------------
fig = plt.figure(figsize=(29.7/2/2.54, 21/2.54),
                 tight_layout=True)

ax = plt.subplot()

ax.set(
       # xlim=(0,120),
       # ylim=(0,180),
       xlabel='Numéro atomique',
       ylabel='Nombre de neutrons',
       )

# titre :
ax.text(0.1, 0.99, 'Modes de transformation des nucléides',
        va='top', transform = ax.transAxes, fontsize=14,
        fontweight='semibold')
# ------------------------------------------------------------
#  ----------------- corps du programme  --------------------
# ------------------------------------------------------------
# bissectrice :
ax.plot([0,120],[0,120], lw=0.4, c='gray', alpha=0.5)

# points :
points_stables(ax)
points('A', ax)
points('B+', ax)
points('B-', ax)
points('EC', ax)
points('SF', ax)
points('n', ax)
points('p', ax)

    # graduations principales et secondaires :
ax.set_xticks(range(0, Zmax, 10))
ax.set_xticks(range(0, Zmax, 2), minor=True)

ymin, ymax = ax.get_ylim()
ymax = int(ymax)
ax.set(
       xlim=(0,120),
       ylim=(0, ymax))
ax.set_yticks(range(0, ymax, 10))
ax.set_yticks(range(0, ymax, 2), minor=True)


ax.legend(markerscale=3,
          title="modes :",
          loc='center left')

# signature :
ax.text(0.99, 0.70, 'D. Alberto (www.astrolabe-science.fr)',
        rotation=90, transform = ax.transAxes, fontsize=6,
        color='dimgray', ha='right')

#  -------------------------------------------------------
#  2e axe inséré : zoom sur les noyaux légers :

xmax = 20
ymax = 30
# cadre sur l'axe principal :
ax.plot([xmax, xmax, 0],[0, ymax, ymax], c='gray', alpha=1,lw=0.4,zorder=0)

ax2 = ax.inset_axes([0.6, 0.03, 0.4, 0.38])  # axe inséré dans le principal
ax2.text(0.99, 0.01, 'Z', ha='right', transform = ax2.transAxes)
ax2.text(0.01, 0.99, 'N', va='top', transform = ax2.transAxes)

ax2.plot([0,120],[0,120], lw=0.4, c='gray', alpha=0.5)

prop_points['s'] = 30  # nouvelle taille des points

# affichage des points :
points_stables(ax2)
points('A', ax2)
points('B+', ax2)
points('B-', ax2)
points('EC', ax2)
points('SF', ax2)
points('n', ax2)
points('p', ax2)

# limites des axes :
ax2.set(
        xlim=(0, xmax),
        ylim=(0, ymax),
        )

# graduations :
ax2.set_xticks(range(0,xmax,2), range(0,xmax,2), fontsize=6)
ax2.set_yticks(range(0,ymax,2), range(0,ymax,2), fontsize=6)
#  -------------------------------------------------------------

fig.savefig('nucleides_modes.png', dpi=450)
fig.savefig('nucleides_modes.pdf')
plt.show()
